#include<dos.h>

#define	AX r.x.ax
#define BX r.x.bx
#define CX r.x.cx
#define DX r.x.dx
#define ES s.es


typedef struct cursor
{
 int screen_mask[16];
 int cursor_mask[16];
} CURSOR;


/* Inicializuje mys/jej driver a vrati pocet podporovanych tlacitok */
/* Ak mys/jej driver niesu nainstalovane vracia nulu                */

int init_mouse(void)
{
 union REGS r;
 AX=0x0000;
 int86(0x33,&r,&r);
 if(AX==0x0000)
  return(0);
 else
  return(BX);
}


/* V zavislosti na rezime zobrazovacej jednotky zapina graficky, */
/* alebo textovy kurzor					   	 */

void show_mouse(void)
{
 union REGS r;
 AX=0x0001;
 int86(0x33,&r,&r);
}


/* Vypina kurzor z obrazovky, no driver sleduje pohyb mysi dalej */

void hide_mouse(void)
{
 union REGS r;
 AX=0x0002;
 int86(0x33,&r,&r);
}


/* Zistuje poziciu mysi a vracia kod stlaceneho tlacitka 0/ziadne */
/* 1/lave, 2/prave, 4/prostredne        			  */
/* Ak je stlacenych viac tlacitok naraz vracia sucet ich kodov    */

int state_mouse(int *x,int *y)
{
 union REGS r;
 AX=0x0003;
 int86(0x33,&r,&r);
 *x=CX;
 *y=DX;
 return(BX);
}


/* Nastavuje aktualnu polohu mysi, pricom obe suradnice musia lezat */
/* v ramci zvoleneho rozsahu, inak su zaokruhlene na najblizsiu     */
/* pripustnu hodnotu						    */

void set_mouse(int x,int y)
{
 union REGS r;
 AX=0x0004;
 CX=x;
 DX=y;
 int86(0x33,&r,&r);
}


/* Cita informacie o stisknuti tlacitka button (0/lave, 1/prave, */
/* 2/stredne), vracia pocet stlaceni testovaneho tlacitka 	 */
/* od posledneho volania tejto funkcie a nastavuje premenne x,y  */
/* na hodnoty suradnic kurzoru pri poslednom stlaceni            */

int read_mouse1(int *x,int *y,int button)
{
 union REGS r;
 AX=0x0005;
 BX=button;
 int86(0x33,&r,&r);
 *x=CX;
 *y=DX;
 return(BX);
}


/* Cita informacie o uvolneni tlacitka button (0/lave, 1/prave,  */
/* 2/stredne), vracia pocet uvolneni testovaneho tlacitka 	 */
/* od posledneho volania tejto funkcie a nastavuje premenne x,y  */
/* na hodnoty suradnic kurzoru pri poslednom uvolneni            */

int read_mouse2(int *x,int *y,int button)
{
 union REGS r;
 AX=0x0006;
 BX=button;
 int86(0x33,&r,&r);
 *x=CX;
 *y=DX;
 return(BX);
}


/* Nastavenie horizontalneho rozsahu suradnic pre pohyb kurzoru */

void mouse_x_range(int xmin, int xmax)
{
 union REGS r;
 AX =0x0007;
 CX = xmin;
 DX = xmax;
 int86(0x33,&r,&r);
}


/* Nastavenie vertikalneho rozsahu suradnic pre pohyb kurzoru */

void mouse_y_range(int ymin, int ymax)
{
 union REGS r;
 AX = 0x0008;
 CX = ymin;
 DX = ymax;
 int86(0x33,&r,&r);
}


/* Definovanie tvaru grafickeho kurzoru so vztaznym bodom x,y */

void set_cursor(int x,int y,CURSOR *c)
{
 union REGS r;
 struct SREGS s;
 AX=0x0009;
 BX=x;
 CX=y;
 ES=FP_SEG(c);
 DX=FP_OFF(c);
 int86x(0x33,&r,&r,&s);
}


/* Nastavenie citlivosti mysi na x_krokov/8 bodov horizontalne */
/* a y_krokov/8 bodov vertikalne                               */

void mouse_sensibility(int x,int y)
{
 union REGS r;
 AX=0x000f;
 CX=x;
 DX=y;
 int86(0x33,&r,&r);
}